<?php

namespace Spec\Infomaker\Everyware\NewsML;

use Infomaker\Everyware\NewsML\Item;
use PhpSpec\Exception\Example\FailureException;
use PhpSpec\ObjectBehavior;
use SimpleXMLIterator;
require_once(__DIR__ . '/xmlStr.php');

class ItemSpec extends ObjectBehavior
{
    private $realisticXML, $simplerXML, $attributes;

    public function let()
    {
        $this->realisticXML = realisticXML;
        $this->simplerXML = simplerXML;
        $this->attributes = attributes;
        $el = simplexml_load_string($this->realisticXML, 'SimpleXMLIterator');
        $this->beConstructedWith($el);
    }
    public function it_is_initializable()
    {
        $this->shouldHaveType(Item::class);
    }
    public function it_should_filter_inline_content()
    {
        $text = '<p>Test paragraph.</p><!-- Comment --> <a href="#fragment">Other text</a>';

        $this->filterInlineContent($text, [])
            ->shouldReturn('<p>Test paragraph.</p><!-- Comment --> <a href="#fragment">Other text</a>');
        $this->filterInlineContent($text, ['a'])
            ->shouldReturn('Test paragraph. <a href="#fragment">Other text</a>');
    }
    public function it_should_get_item_attribute()
    {
        $xml = '
            <xml stuff="it works!">
            </xml>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->beConstructedWith($el);
        $this->getItemAttribute('stuff')
            ->shouldReturn('it works!');
    }
    public function it_should_return_a_default_value_if_item_attribute_is_not_found()
    {
        $this->getItemAttribute('notFound', 'defaultValue')
            ->shouldReturn('defaultValue');
    }
    public function it_should_return_a_child()
    {
        $this->getFirstChild('links')
            ->shouldHaveArrayKey('link');
    }
    public function it_should_get_type()
    {
        $xml = '
            <xml type="x-im/content-part">
            </xml>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->beConstructedWith($el);
        $this->getType()
            ->shouldReturn('x-im/content-part');
    }
    public function it_should_remove_all_line_breaks()
    {
        $this->removeLinebreak($this->simplerXML)
            ->shouldReturn('<movies><movie>abcd</movie><movie>efgh</movie><movie>hijk</movie></movies>');
    }
    public function it_should_get_content_as_xml()
    {
        $el = new SimpleXmlIterator($this->simplerXML, null);
        $this->beConstructedWith($el);
        $this->getContentAsXML()
            ->shouldReturn('<?xml version="1.0"?><movies><movie>abcd</movie><movie>efgh</movie><movie>hijk</movie></movies>');
    }
    public function it_should_return_the_first_child()
    {
        $xml = '
        
            <link rel="self" type="x-im/image" uri="im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg" uuid="d83dad15-9476-54c1-a57d-495757664dae">
                <data>
                    <width>300</width>
                    <height>169</height>
                    <alttext/>
                </data>
                <data>
                    <width>3000</width>
                    <height>1690</height>
                    <alttext/>
                </data>
            </link>
        
            ';
        $el = new SimpleXmlIterator($xml, null);
        $this->beConstructedWith($el);
        $this->getFirstChild('data')
            ->shouldHaveKeysValues([
                'width' => '300',
                'height' => '169',
            ]);
    }
    public function it_should_return_all_children_with_values()
    {
        $xml = '
        
            <link rel="self" type="x-im/image" uri="im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg" uuid="d83dad15-9476-54c1-a57d-495757664dae">
                <data>
                    <width>300</width>
                    <height>169</height>
                    <alttext/>
                </data>
                <data>
                    <width>3000</width>
                    <height>1690</height>
                    <alttext/>
                </data>
            </link>
        
            ';
        $el = new SimpleXmlIterator($xml, null);
        $this->beConstructedWith($el);
        $this->getChild('data')
            ->shouldHaveChildren([
                [
                    'width' => '300',
                    'height' => '169',
                ],
                [
                    'width' => '3000',
                    'height' => '1690',
                ],
            ]);
    }
    public function it_should_get_filtered_content()
    {
        $this->getFilteredContent(['attributes', 'name', 'game', 'type', 'title', 'data'])
            ->shouldReturn('<attributes id="MjQ0LDI4LDIxNSwxNzk" title="NEW Fancy teaser with fancy article with fancy stuff" type="x-im/teaser"/><attributes name="one" game="lonely"><data>35202347</data></attributes>');
    }
    public function it_should_return_first_child_as_xml()
    {
        $xml = '
            <data>
                <text>heheh</text>
            </data>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->beConstructedWith($el);
        $this->getFirstChildAsXml('text')
            ->shouldEqual('<text>heheh</text>');
    }
    public function it_should_return_first_child_as_xml_with_allowed_tags()
    {
        $xml = '
            <data>
                <text><em>hey</em><strong>there</strong><mark>general</mark><a href="#">kenobi</a></text>
            </data>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->beConstructedWith($el);
        $this->getFirstChildAsXml('text', ['mark', 'em', 'strong', 'a'])
            ->shouldEqual('<em>hey</em><strong>there</strong><mark>general</mark><a href="#">kenobi</a>');
    }
    public function it_should_return_first_child_as_xml_with_default_value()
    {
        $xml = '<data></data>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->beConstructedWith($el);
        $this->getFirstChildAsXml('text')
            ->shouldEqual('');
    }
    public function getMatchers(): array
    {
        return [
            'haveKeys' => function ($subject, $keys) {
                if (count(array_intersect_key(array_flip($keys), $subject)) === count($keys)) {
                    return true;
                }
            },
            'haveValue' => function (array $subject, string $key, $value) {
                return array_filter($subject, function($item) use($value, $key){
                    return ($item->$key === $value);
                 });
            },
            'haveChildren' => function ($subject, $values) {
                foreach($subject as $index => $item) {
                    if($item->getProperties() != $values[$index])
                        return false;
                }

                return true;
            },
            'haveKeysValues' => function ($subject, $key) {
                return $subject->getProperties() == $key;
            },
            'haveArrayKey' => function ($subject, $key) {
                if (!array_key_exists($key, $subject->getProperties())) {
                    throw new FailureException(sprintf(
                        'Message with subject "%s" and key "%s".',
                        array_shift($subject),
                        $key
                    ));
                }
                return true;
            },
        ];
    }
}
